/**
 * @file
 * Viewfield Argument Helper autocomplete functionality.
 */

(function (Drupal, drupalSettings, once) {
  'use strict';

  /**
   * Creates autocomplete functionality for a slot.
   * 
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   */
  const createAutocomplete = (parent, slot) => {
    const target_id = parent.querySelector('select[name$="[target_id]"]')?.value;
    const display_id = parent.querySelector('select[name$="[display_id]"]')?.value;
    if (!target_id || !display_id) {
      return;
    }

    // Clear existing content to prevent duplicates
    const existingContent = slot.querySelectorAll('.viewfield-argument-helper__element');
    existingContent.forEach(element => element.remove());

    // Fetch slot information and create autocomplete for each.
    fetchSlotInfo(target_id, display_id)
      .then(slotInfo => {
        if (slotInfo.length == 0) {
          showErrorMessage(slot, Drupal.t('No contextual arguments found on this display.'));
          return;
        }
        showErrorMessage(slot, ''); // Clear any previous error messages

        // Open the Advanced options details.
        parent.querySelector('details').setAttribute('open', 'open');

        Object.values(slotInfo).forEach(definition => {
          createAutocompleteForSlot(parent, slot, target_id, display_id, definition);
        });

        createUpdateButton(parent, slot);
      })
      .catch(error => {
        console.error('Error fetching slot info:', error);
        showErrorMessage(slot, Drupal.t('Failed to load slot information.'));
      });
  };

  /**
   * Creates an autocomplete select element for a specific slot.
   * 
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   * @param {string} targetId - The target view ID.
   * @param {string} displayId - The display ID.
   * @param {Object} definition - The slot definition object.
   */
  const createAutocompleteForSlot = (parent, slot, targetId, displayId, definition) => {
    const container = document.createElement('div');
    container.classList.add('viewfield-argument-helper__slot-container', 'viewfield-argument-helper__element');

    // Unique identifier for slot + argument.
    const id = parent.id + '-' + definition.id;

    // Create the select input
    const input = document.createElement('select');
    input.classList.add('slot-select', 'viewfield-argument-helper__slot-select');
    input.setAttribute('data-name', definition.name);
    input.setAttribute('data-target-id', targetId);
    input.setAttribute('data-display-id', displayId);
    input.id = id
    
    if (definition.is_multiple) {
      input.setAttribute('multiple', 'multiple');
    }

    // Create the label
    const label = document.createElement('label');
    label.textContent = Drupal.t('Slot @number: @name', {
      '@number': definition.i_slot + 1,
      '@name': definition.name
    });
    label.setAttribute('for', id);
    label.classList.add('form-item__label', 'viewfield-argument-helper__slot-label');

    // Create AND/OR checkbox for multiple selections
    if (definition.is_multiple) {
      const andOrContainer = createAndOrCheckbox(id);
      container.appendChild(andOrContainer);
    }

    container.appendChild(label);
    container.appendChild(input);
    slot.appendChild(container);

    // Initialize Choices.js
    initializeChoicesJs(input, targetId, displayId, definition, parent, slot);
  };

  /**
   * Generates the argument string from selected values.
   * 
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   */
  const makeArgumentString = (parent, slot) => {
    const selects = slot.querySelectorAll('select.slot-select');
    const values = [];

    selects.forEach((select) => {
      const selectedValues = Array.from(select.options)
        .filter(option => option.hasAttribute('selected'))
        .map(option => option.value);

      if (selectedValues.length === 0) {
        values.push('all');
      } else {
        const andOrCheckbox = document.getElementById(`${select.id}-andor`);
        const joiner = (andOrCheckbox && andOrCheckbox.checked) ? ',' : '+';
        values.push(selectedValues.join(joiner));
      }
    });

    const argumentInput = parent.querySelector('[name*="[arguments]"]');
    if (argumentInput) {
      argumentInput.value = values.join('/');
      
      // Trigger change event for other scripts that might be listening
      argumentInput.dispatchEvent(new Event('change', { bubbles: true }));
    }
  };

  /**
   * Parses the argument string into an array of argument arrays.
   * 
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   * @returns {Array<Array<string>>} Array of argument arrays.
   */
  const parseArguments = (parent, slot) => {
    const argumentInput = parent.querySelector('input[name$="[arguments]"]');
    if (!argumentInput || !argumentInput.value) {
      return [];
    }

    const args = argumentInput.value.split('/');
    const selects = slot.querySelectorAll('select.slot-select');

    // Ensure we have an array of the correct length
    while (args.length < selects.length) {
      args.push('');
    }

    // Parse each argument and update AND/OR checkboxes
    return args.map((arg, index) => {
      if (!arg) {
        return [''];
      }

      const isAnd = arg.includes(',');
      const joiner = isAnd ? ',' : '+';
      
      // Update the corresponding AND/OR checkbox
      if (selects[index]) {
        const andOrCheckbox = document.getElementById(`${selects[index].id}-andor`);
        if (andOrCheckbox) {
          andOrCheckbox.checked = isAnd;
        }
      }

      return arg.split(new RegExp(`[${joiner}]`)).filter(val => val !== '');
    });
  };

  /**
   * Fetches slot information from the server.
   * 
   * @param {string} targetId - The target view ID.
   * @param {string} displayId - The display ID.
   * @returns {Promise<Object>} Promise resolving to slot info.
   */
  const fetchSlotInfo = async (targetId, displayId) => {
    const url = `${drupalSettings.path.baseUrl}viewfield_argument_helper/slotinfo/${targetId}/${displayId}`;
    const response = await fetch(url);
    
    if (!response.ok) {
      throw new Error(`HTTP error! status: ${response.status}`);
    }
    
    return response.json();
  };

  /**
   * Creates the update button if it doesn't exist.
   * 
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   */
  const createUpdateButton = (parent, slot) => {
    if (slot.querySelector('.viewfield-argument-helper__update-button')) {
      return;
    }

    const updateButton = document.createElement('button');
    updateButton.classList.add(
      'button', 
      'button--primary', 
      'button--small',
      'viewfield-argument-helper__update-button',
      'viewfield-argument-helper__element'
    );
    updateButton.type = 'button';
    updateButton.textContent = Drupal.t('Update Arguments ↓');
    updateButton.title = Drupal.t('Update the contextual argument string below based on selected values.');
    
    updateButton.addEventListener('click', (event) => {
      event.preventDefault();
      makeArgumentString(parent, slot);
    });
    
    slot.appendChild(updateButton);
  };

  /**
   * Shows an error message to the user.
   * 
   * @param {HTMLElement} container - Container to show error in.
   * @param {string} message - Error message to display.
   */
  const showErrorMessage = (container, message) => {
     // Clear any existing error messages
    const existingError = container.querySelector('.messages');
    if (existingError) {
      existingError.remove();
    }
    if (message == '') {
      return;
    }

    const errorDiv = document.createElement('div');
    errorDiv.classList.add('messages', 'messages--error');
    errorDiv.textContent = message;
    container.appendChild(errorDiv);
  };

  /**
   * Creates the AND/OR checkbox for multiple selection slots.
   * 
   * @param {string} selectId - The ID of the select element.
   * @returns {HTMLElement} The AND/OR container element.
   */
  const createAndOrCheckbox = (selectId) => {
    const andOrId = `${selectId}-andor`;
    const container = document.createElement('div');
    container.classList.add('viewfield-argument-helper__andor-container');
    
    const checkbox = document.createElement('input');
    checkbox.id = andOrId;
    checkbox.type = 'checkbox';
    checkbox.classList.add('viewfield-argument-helper__andor-checkbox');
    
    const label = document.createElement('label');
    label.setAttribute('for', andOrId);
    label.textContent = Drupal.t('AND');
    label.classList.add('viewfield-argument-helper__andor-label');
    label.title = Drupal.t('Override default OR (union) behavior and use AND (intersection) of selections.');
    
    container.appendChild(checkbox);
    container.appendChild(label);
    
    return container;
  };

  /**
   * Initializes Choices.js for the given select element.
   * 
   * @param {HTMLSelectElement} input - The select input element.
   * @param {string} targetId - The target view ID.
   * @param {string} displayId - The display ID.
   * @param {Object} definition - The slot definition object.
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   */
  const initializeChoicesJs = (input, targetId, displayId, definition, parent, slot) => {
    const choices = new Choices(input, {
      allowHTML: true,
      removeItemButton: true,
      loadingText: Drupal.t('Loading...'),
    });

    // Load choices asynchronously
    loadChoices(choices, targetId, displayId, definition, parent, slot);
  };

  /**
   * Loads choices for a Choices.js instance.
   * 
   * @param {Choices} choices - The Choices.js instance.
   * @param {string} targetId - The target view ID.
   * @param {string} displayId - The display ID.
   * @param {Object} definition - The slot definition object.
   * @param {HTMLElement} parent - The parent fieldset element.
   * @param {HTMLElement} slot - The slot div element.
   */
  const loadChoices = async (choices, targetId, displayId, definition, parent, slot) => {
    try {
      const url = `${drupalSettings.path.baseUrl}viewfield_argument_helper/autocomplete/${targetId}/${displayId}/${definition.id}`;
      const response = await fetch(url);
      
      if (!response.ok) {
        throw new Error(`HTTP error! status: ${response.status}`);
      }
      
      const options = await response.json();
      const parsedArguments = parseArguments(parent, slot);
      
      // Mark options as selected if they match parsed arguments
      if (parsedArguments[definition.i_slot] && parsedArguments[definition.i_slot][0] !== 'all') {
        options.forEach(option => {
          if (parsedArguments[definition.i_slot].some(val => String(val) === String(option.value))) {
            option.selected = true;
          }
        });
      }
      
      choices.setChoices(options, 'value', 'label', true);
    } catch (error) {
      console.error('Error loading choices:', error);
      choices.setChoices([{
        value: '',
        label: Drupal.t('Error loading options'),
        disabled: true
      }], 'value', 'label', true);
    }
  };

  /**
   * Attaches the autocomplete functionality to the input field.
   */
  Drupal.behaviors.viewfieldArgumentHelperAutocomplete = {
    attach(context, settings) {
      // Ensure the Choices library is loaded.
      if (typeof Choices === 'undefined') {
        console.warn('Choices.js library is not loaded. Autocomplete functionality will not be available.');
        return;
      }

      // Use once() to prevent duplicate initialization
      const slotDivs = once('viewfield-argument-helper-autocomplete', '.vaa-slots > div', context);

      slotDivs.forEach((slot) => {
        // Outer wrapper for the field.
        const parent = slot.closest('div.field--type-viewfield');
        if (!parent) {
          console.warn('Could not find viewfield parent for slot element.');
          return;
        }

        createAutocomplete(parent, slot);

        // Detect changes to the target_id and display_id selects.
        const selects = parent.querySelectorAll('select.form-select');
        selects.forEach((select) => {
          select.addEventListener('change', () => {
            createAutocomplete(parent, slot);
            // Try to get display_id after a short delay.
            if (!parent.querySelector('select[name$="[display_id]"]')?.value) {
              setTimeout(() => {
                createAutocomplete(parent, slot);
              }, 500);
            }
          });
        });
      });
    },

    /**
     * Cleanup function when behavior is detached.
     */
    detach(context, settings, trigger) {
      if (trigger === 'unload') {
        // Cleanup any Choices.js instances or event listeners if needed
        const choicesElements = context.querySelectorAll('.slot-select');
        choicesElements.forEach((element) => {
          if (element.choices) {
            element.choices.destroy();
          }
        });
      }
    }
  };

})(Drupal, drupalSettings, once);